from github import Auth
from github import Github
import os
import sys
import random
import datetime
import time
import logging
import base64

logging.basicConfig(
    format="%(asctime)s.%(msecs)d %(levelname)-8s"
    + " [%(filename)s:%(lineno)d] %(message)s",
    datefmt="%Y-%m-%d:%H:%M:%S",
    level=logging.INFO,
)

# Scan parameters
SCAN_IMAGE_LIST = os.environ.get("SCAN_IMAGE_LIST")

# GitHub Settings
GITHUB_SERVER = os.environ.get("GITHUB_SERVER", "https://github.gwd.broadcom.net")
REPO = "SED/ESG-ProdSec-Scanner"
BRANCH = "main"
WORKFLOW_FILE = "scan.yml"

# GitHub App config
CLIENT_ID = os.environ.get("APP_CLIENT_ID", 430)
INSTALLATION_ID = os.environ.get("APP_INSTALLATION_ID", 809)
PRIVATE_KEY_B64 = os.environ.get("PRIVATE_KEY_B64", "")
PRIVATE_KEY = base64.b64decode(PRIVATE_KEY_B64).decode()

MAX_AGE_MINUTES = 10
RETRIES = 6
DELAY_SECONDS = 30

EXIT_NOT_TRIGGERED = 101
EXIT_RUN_ID_NOT_FOUND = 102
EXIT_RUN_FAIL = 103


def get_token():
    logging.info(f"Generating token for app id: {CLIENT_ID} and installation id: {INSTALLATION_ID}")
    auth = Auth.AppAuth(CLIENT_ID, PRIVATE_KEY).get_installation_auth(INSTALLATION_ID)
    github = Github(auth=auth, base_url=f"{GITHUB_SERVER}/api/v3")
    return auth.token


def connect_to_github():
    logging.info("Logging using the generated token")
    token = get_token()
    auth = Auth.Token(token)
    g = Github(auth=auth, base_url=f"{GITHUB_SERVER}/api/v3")

    return g


def trigger_workflow_run(wf, SCAN_IMAGE_LIST, CUSTOM_ID):

    logging.info(f"Triggering workflow {wf} with CUSTOM_ID='{CUSTOM_ID}' for '{SCAN_IMAGE_LIST}'")

    return wf.create_dispatch(
        ref=BRANCH,
        inputs={
            "SCAN_IMAGE_LIST_INPUT": SCAN_IMAGE_LIST,
            "CUSTOM_ID": f"{CUSTOM_ID}"
        }
    )


def get_workflow_run(wf, CUSTOM_ID):

    logging.info(f"Searching for workflow run with CUSTOM_ID='{CUSTOM_ID}'")

    now = datetime.datetime.now()
    no_sooner_than = now - datetime.timedelta(minutes=MAX_AGE_MINUTES)
    runs = wf.get_runs(branch=BRANCH, created=f">{no_sooner_than.isoformat().split('.')[0]}")

    for run in runs:
        try:
            steps = run.jobs()[0].steps
            if steps[1].name == f"id={CUSTOM_ID}":
                logging.info(f"Found workflow run with id: {run.id}")
                return run
        except Exception as e:
            pass

    logging.info(f"Workflow run not found (yet)")
    return None


if __name__ == "__main__":
    g = connect_to_github()

    r = g.get_repo(REPO)

    wf = r.get_workflow(WORKFLOW_FILE)

    CUSTOM_ID = random.randbytes(16).hex()

    # Trigger workflow run
    if trigger_workflow_run(wf, SCAN_IMAGE_LIST, CUSTOM_ID) is False:
        logging.error(f"Failed to trigger workflow {wf} with CUSTOM_ID='{CUSTOM_ID}' for '{SCAN_IMAGE_LIST}'")
        sys.exit(EXIT_NOT_TRIGGERED)

    for i in range(RETRIES):
        # Get workflow run
        run = get_workflow_run(wf, CUSTOM_ID)

        if run is not None:
            break

        logging.info(f"Waiting for {DELAY_SECONDS} seconds before trying again")
        time.sleep(DELAY_SECONDS)

    if run is None:
        logging.error("Workflow run not found!")
        sys.exit(EXIT_RUN_ID_NOT_FOUND)

    # Poll for status until completion
    logging.info(f"Polling for completion every {DELAY_SECONDS} seconds")
    while run.status != "completed":
        time.sleep(DELAY_SECONDS)
        run.update()

    print("\n\n\n")
    print(f"Workflow completed with status: {run.conclusion}")
    print(f"Workflow run URL: {GITHUB_SERVER}/{REPO}/actions/runs/{run.id}")
    print("\n\n\n")

    if run.conclusion != "success":
        sys.exit(EXIT_RUN_FAIL)
